import json
import os
from aiohttp import web

from configparser import ConfigParser
from services.basicservice import BasicService
from singleton import Singleton

class IKassa(BasicService, metaclass=Singleton):
    def __init__(self, servicePath: str):
        super().__init__(__file__)
        self.servicePath = servicePath
        self._loadAllCases()

    def getRoutes(self) -> list[web.AbstractRouteDef]:
        return [web.post(self.servicePath + '/ik.service.app/version', handler=self.version),
                web.post(self.servicePath, handler=self.entryPoint)]

    async def version(self, request: web.Request) -> web.Response:
        self.log.info("Получение версии протокола ikassa")
        return self._createResponse('/ik.service.app/version')

    async def entryPoint(self, request: web.Request) -> web.Response:
        self.log.info("Получен запрос")
        request_data = await request.json()
        command = str(request_data['headers']['action']).lower()
        self.log.info(f"Метод '{command}'")
        return self._createResponse(f'/{command}')

class IKassaDynamic(IKassa):
    def __init__(self, servicePath: str):
        super().__init__(servicePath)
        self.methods = {
            'get_token_state': "get_token_state_closed",
            'get_x_report': "get_x_report"
        }
        self.commandMap = {
            "open_shift": self.onOpenShift,
            "close_shift": self.onCloseShift,
            "get_token_state": self.onGetTokenState,
            "create_deposit": self.onCreateDeposit,
            "create_withdraw": self.onCreateWithdraw,
            "create_sale": self.onCreateSale,
            "create_money_back": self.onCreateMoneyBack,
            "get_x_report": self.onGetXReport
        }

    async def entryPoint(self, request: web.Request) -> web.Response:
        self.log.info("Получен запрос")
        request_data = await request.json()
        command = str(request_data['headers']['action']).lower()
        self.log.info(f"Метод '{command}'")
        return self.processCommand(command, request_data)

    def processCommand(self, command: str, request_data: dict) -> web.Response:
        handler = self.commandMap.get(command)
        if not handler:
            return self._createResponse(f'/{command}')
        return handler(request_data)

    def onOpenShift(self, request_data: dict) -> web.Response:
        self.methods['get_token_state'] = "get_token_state"
        return self._createResponse('/open_shift')

    def onCloseShift(self, request_data: dict) -> web.Response:
        self.methods['get_token_state'] = "get_token_state_closed"
        return self._createResponse('/close_shift')

    def onGetTokenState(self, request_data: dict) -> web.Response:
        command = self.methods.get('get_token_state', 'get_token_state_closed')
        return self._createResponse(f'/{command}')

    def onCreateDeposit(self, request_data: dict) -> web.Response:
        response = self._createResponse('/create_deposit')
        data = json.loads(response.text)
        data['data']['sum'] = request_data['data']['sum_cheque_data']['sum']
        response.text = json.dumps(data)
        return response

    def onCreateWithdraw(self, request_data: dict) -> web.Response:
        response = self._createResponse('/create_withdraw')
        data = json.loads(response.text)
        data['data']['sum'] = request_data['data']['sum_cheque_data']['sum']
        response.text = json.dumps(data)
        return response

    def onCreateSale(self, request_data: dict) -> web.Response:
        self.methods['get_x_report'] = "get_x_report_sale"
        return self._createResponse('/create_sale')

    def onCreateMoneyBack(self, request_data: dict) -> web.Response:
        self.methods['get_x_report'] = "get_x_report_moneyback"
        return self._createResponse('/create_money_back')

    def onGetXReport(self, request_data: dict) -> web.Response:
        command = self.methods.get('get_x_report', 'get_x_report')
        response = self._createResponse(f'/{command}')
        return response


conf = ConfigParser()
conf.read(os.path.join(os.path.dirname(__file__), 'config.ini'))
servicePath = conf.get('ikassa', 'path') if conf.has_option('ikassa', 'path') else '/ikassa'
dynamic = conf.getboolean('ikassa', 'dynamic', fallback = False)
instance = IKassaDynamic(servicePath) if dynamic else IKassa(servicePath)
